@@ -11,11 +11,13 @@ void enable_arm64() {}
 
 static cs_err init(cs_struct *ud)
 {
+	MCRegisterInfo *mri;
+
 	// verify if requested mode is valid
 	if (ud->mode & ~(CS_MODE_LITTLE_ENDIAN | CS_MODE_ARM | CS_MODE_BIG_ENDIAN))
 		return CS_ERR_MODE;
 
-	MCRegisterInfo *mri = cs_mem_malloc(sizeof(*mri));
+	mri = cs_mem_malloc(sizeof(*mri));
 
 	AArch64_init(mri);
 	ud->printer = AArch64_printInst;
